from enum import Enum


class LogType(str, Enum):
    AWS_ALB = "AWS.ALB"
    AWS_AURORA_MYSQL_AUDIT = "AWS.AuroraMySQLAudit"
    AWS_CLOUDTRAIL = "AWS.CloudTrail"
    AWS_CLOUDTRAIL_DIGEST = "AWS.CloudTrailDigest"
    AWS_CLOUDTRAIL_INSIGHT = "AWS.CloudTrailInsight"
    AWS_CLOUDWATCH_EVENTS = "AWS.CloudWatchEvents"
    AWS_CONFIG = "AWS.Config"
    AWS_GUARDDUTY = "AWS.GuardDuty"
    AWS_S3_SERVER_ACCESS = "AWS.S3ServerAccess"
    AWS_SECURITY_FINDING_FORMAT = "AWS.SecurityFindingFormat"
    AWS_TRANSIT_GATEWAY_FLOW = "AWS.TransitGatewayFlow"
    AWS_VPC_DNS = "AWS.VPCDns"
    AWS_VPC_FLOW = "AWS.VPCFlow"
    AWS_WAF_WEB_ACL = "AWS.WAFWebACL"
    ALPHASOC_ALERT = "AlphaSOC.Alert"
    AMAZON_EKS_AUDIT = "Amazon.EKS.Audit"
    AMAZON_EKS_AUTHENTICATOR = "Amazon.EKS.Authenticator"
    ANOMALI_INDICATOR = "Anomali.Indicator"
    APACHE_ACCESS_COMBINED = "Apache.AccessCombined"
    APACHE_ACCESS_COMMON = "Apache.AccessCommon"
    APPOMNI_ALERTS = "AppOmni.Alerts"
    APPOMNI_EVENTS = "AppOmni.Events"
    APPOMNI_POLICY = "AppOmni.Policy"
    ASANA_AUDIT = "Asana.Audit"
    ATLASSIAN_AUDIT = "Atlassian.Audit"
    AUTH0_EVENTS = "Auth0.Events"
    AZURE_AUDIT = "Azure.Audit"
    AZURE_DEFENDER_ALERTS = "Azure.DefenderAlerts"
    AZURE_MONITOR_ACTIVITY = "Azure.MonitorActivity"
    BITWARDEN_EVENTS = "Bitwarden.Events"
    BOX_EVENT = "Box.Event"
    CARBON_BLACK_ALERT_V2 = "CarbonBlack.AlertV2"
    CARBON_BLACK_AUDIT = "CarbonBlack.Audit"
    CARBON_BLACK_ENDPOINT_EVENT = "CarbonBlack.EndpointEvent"
    CARBON_BLACK_WATCH_LIST_HIT = "CarbonBlack.WatchlistHit"
    CISCO_UMBRELLA_CLOUD_FIREWALL = "CiscoUmbrella.CloudFirewall"
    CISCO_UMBRELLA_DNS = "CiscoUmbrella.DNS"
    CISCO_UMBRELLA_IP = "CiscoUmbrella.IP"
    CISCO_UMBRELLA_PROXY = "CiscoUmbrella.Proxy"
    CLOUDFLARE_AUDIT = "Cloudflare.Audit"
    CLOUDFLARE_FIREWALL = "Cloudflare.Firewall"
    CLOUDFLARE_HTTP_REQUEST = "Cloudflare.HttpRequest"
    CLOUDFLARE_SPECTRUM = "Cloudflare.Spectrum"
    CLOUDFLARE_ZERO_TRUST_RDATA = "Cloudflare.ZeroTrust.RData"
    CROWDSTRIKE_AID_MASTER = "Crowdstrike.AIDMaster"
    CROWDSTRIKE_ACTIVITY_AUDIT = "Crowdstrike.ActivityAudit"
    CROWDSTRIKE_APP_INFO = "Crowdstrike.AppInfo"
    CROWDSTRIKE_CRITICAL_FILE = "Crowdstrike.CriticalFile"
    CROWDSTRIKE_DNS_REQUEST = "Crowdstrike.DNSRequest"
    CROWDSTRIKE_DETECTION_SUMMARY = "Crowdstrike.DetectionSummary"
    CROWDSTRIKE_EVENT_STREAMS = "Crowdstrike.EventStreams"
    CROWDSTRIKE_FDR_EVENT = "Crowdstrike.FDREvent"
    CROWDSTRIKE_GROUP_IDENTITY = "Crowdstrike.GroupIdentity"
    CROWDSTRIKE_MANAGED_ASSETS = "Crowdstrike.ManagedAssets"
    CROWDSTRIKE_NETWORK_CONNECT = "Crowdstrike.NetworkConnect"
    CROWDSTRIKE_NETWORK_LISTEN = "Crowdstrike.NetworkListen"
    CROWDSTRIKE_NOT_MANAGED_ASSETS = "Crowdstrike.NotManagedAssets"
    CROWDSTRIKE_PROCESS_ROLLUP_2 = "Crowdstrike.ProcessRollup2"
    CROWDSTRIKE_PROCESS_ROLLUP_2_STATS = "Crowdstrike.ProcessRollup2Stats"
    CROWDSTRIKE_SYNTHETIC_PROCESS_ROLLUP_2 = "Crowdstrike.SyntheticProcessRollup2"
    CROWDSTRIKE_UNKNOWN = "Crowdstrike.Unknown"
    CROWDSTRIKE_USER_IDENTITY = "Crowdstrike.UserIdentity"
    CROWDSTRIKE_USER_INFO = "Crowdstrike.UserInfo"
    CROWDSTRIKE_USER_LOGON_LOGOFF = "Crowdstrike.UserLogonLogoff"
    DOCKER_EVENTS = "Docker.Events"
    DROPBOX_TEAM_EVENT = "Dropbox.TeamEvent"
    DUO_ADMINISTRATOR = "Duo.Administrator"
    DUO_AUTHENTICATION = "Duo.Authentication"
    DUO_OFFLINE_ENROLLMENT = "Duo.OfflineEnrollment"
    DUO_TELEPHONY = "Duo.Telephony"
    ENVOY_ACCESS = "Envoy.Access"
    FASTLY_ACCESS = "Fastly.Access"
    FLUENTD_SYSLOG3164 = "Fluentd.Syslog3164"
    FLUENTD_SYSLOG5424 = "Fluentd.Syslog5424"
    GCP_AUDIT_LOG = "GCP.AuditLog"
    GCP_HTTP_LOAD_BALANCER = "GCP.HTTPLoadBalancer"
    GSUITE_ACTIVITY_EVENT = "GSuite.ActivityEvent"
    GSUITE_DIRECTORY_USERS = "GSuite.DirectoryUsers"
    GSUITE_REPORTS = "GSuite.Reports"
    GITHUB_AUDIT = "GitHub.Audit"
    GITHUB_WEBHOOK = "GitHub.Webhook"
    GITLAB_API = "GitLab.API"
    GITLAB_AUDIT = "GitLab.Audit"
    GITLAB_EXCEPTIONS = "GitLab.Exceptions"
    GITLAB_GIT = "GitLab.Git"
    GITLAB_INTEGRATIONS = "GitLab.Integrations"
    GITLAB_PRODUCTION = "GitLab.Production"
    GRAVITATIONAL_TELEPORT_AUDIT = "Gravitational.TeleportAudit"
    GREYNOISE_NOISE = "GreyNoise.Noise"
    GREYNOISE_RIOT = "GreyNoise.RIOT"
    HEROKU_RUNTIME = "Heroku.Runtime"
    IPINFO_ASN_CIDR = "IPInfo.ASNCIDR"
    IPINFO_ASN_RANGES = "IPInfo.ASNRanges"
    IPINFO_LOCATION_CIDR = "IPInfo.LocationCIDR"
    IPINFO_LOCATION_RANGES = "IPInfo.LocationRanges"
    IPINFO_PRIVACY_CIDR = "IPInfo.PrivacyCIDR"
    IPINFO_PRIVACY_RANGES = "IPInfo.PrivacyRanges"
    JAMF_PRO_COMPLIANCE_REPORTER = "Jamfpro.ComplianceReporter"
    JAMF_PRO_LOGIN = "Jamfpro.Login"
    JUNIPER_ACCESS = "Juniper.Access"
    JUNIPER_AUDIT = "Juniper.Audit"
    JUNIPER_FIREWALL = "Juniper.Firewall"
    JUNIPER_MWS = "Juniper.MWS"
    JUNIPER_POSTGRES = "Juniper.Postgres"
    JUNIPER_SECURITY = "Juniper.Security"
    LACEWORK_AGENT_MANAGEMENT = "Lacework.AgentManagement"
    LACEWORK_ALERT_DETAILS = "Lacework.AlertDetails"
    LACEWORK_ALL_FILES = "Lacework.AllFiles"
    LACEWORK_APPLICATIONS = "Lacework.Applications"
    LACEWORK_CHANGE_FILES = "Lacework.ChangeFiles"
    LACEWORK_CLOUD_COMPLIANCE = "Lacework.CloudCompliance"
    LACEWORK_CLOUD_CONFIGURATION = "Lacework.CloudConfiguration"
    LACEWORK_CMD_LINE = "Lacework.Cmdline"
    LACEWORK_CONNECTIONS = "Lacework.Connections"
    LACEWORK_CONTAINER_SUMMARY = "Lacework.ContainerSummary"
    LACEWORK_CONTAINER_VULN_DETAILS = "Lacework.ContainerVulnDetails"
    LACEWORK_DNS_QUERY = "Lacework.DNSQuery"
    LACEWORK_EVENTS = "Lacework.Events"
    LACEWORK_HOST_VULN_DETAILS = "Lacework.HostVulnDetails"
    LACEWORK_IMAGE = "Lacework.Image"
    LACEWORK_INTERFACES = "Lacework.Interfaces"
    LACEWORK_INTERNAL_IPA = "Lacework.InternalIPA"
    LACEWORK_MACHINE_DETAILS = "Lacework.MachineDetails"
    LACEWORK_MACHINE_SUMMARY = "Lacework.MachineSummary"
    LACEWORK_NEW_HASHES = "Lacework.NewHashes"
    LACEWORK_PACKAGE = "Lacework.Package"
    LACEWORK_POD_SUMMARY = "Lacework.PodSummary"
    LACEWORK_PROCESS_SUMMARY = "Lacework.ProcessSummary"
    LACEWORK_USER_DETAILS = "Lacework.UserDetails"
    LACEWORK_USER_LOGIN = "Lacework.UserLogin"
    LINUX_AUDITD = "Linux.Auditd"
    MICROSOFT365_AUDIT_AZURE_ACTIVE_DIRECTORY = "Microsoft365.Audit.AzureActiveDirectory"
    MICROSOFT365_AUDIT_EXCHANGE = "Microsoft365.Audit.Exchange"
    MICROSOFT365_AUDIT_GENERAL = "Microsoft365.Audit.General"
    MICROSOFT365_AUDIT_SHAREPOINT = "Microsoft365.Audit.SharePoint"
    MICROSOFT365_DLP_ALL = "Microsoft365.DLP.All"
    MICROSOFT_GRAPH_SECURITY_ALERT = "MicrosoftGraph.SecurityAlert"
    MONGODB_ORGANIZATION_EVENT = "MongoDB.OrganizationEvent"
    MONGODB_PROJECT_EVENT = "MongoDB.ProjectEvent"
    NETSKOPE_AUDIT = "Netskope.Audit"
    NGINX_ACCESS = "Nginx.Access"
    NGINX_ERROR = "Nginx.Error"
    NOTION_AUDIT_LOGS = "Notion.AuditLogs"
    OCSF_ACCOUNT_CHANGE = "OCSF.AccountChange"
    OCSF_API_ACTIVITY = "OCSF.ApiActivity"
    OCSF_APPLICATION_LIFECYCLE = "OCSF.ApplicationLifecycle"
    OCSF_AUTHENTICATION = "OCSF.Authentication"
    OCSF_AUTHORIZE_SESSION = "OCSF.AuthorizeSession"
    OCSF_BASE_EVENT = "OCSF.BaseEvent"
    OCSF_COMPLIANCE_FINDING = "OCSF.ComplianceFinding"
    OCSF_CONFIG_STATE = "OCSF.ConfigState"
    OCSF_DATASTORE_ACTIVITY = "OCSF.DatastoreActivity"
    OCSF_DETECTION_FINDING = "OCSF.DetectionFinding"
    OCSF_DEVICE_CONFIG_STATE_CHANGE = "OCSF.DeviceConfigStateChange"
    OCSF_DHCP_ACTIVITY = "OCSF.DhcpActivity"
    OCSF_DNS_ACTIVITY = "OCSF.DnsActivity"
    OCSF_EMAIL_ACTIVITY = "OCSF.EmailActivity"
    OCSF_EMAIL_FILE_ACTIVITY = "OCSF.EmailFileActivity"
    OCSF_EMAIL_URL_ACTIVITY = "OCSF.EmailUrlActivity"
    OCSF_ENTITY_MANAGEMENT = "OCSF.EntityManagement"
    OCSF_FILE_ACTIVITY = "OCSF.FileActivity"
    OCSF_FILE_HOSTING = "OCSF.FileHosting"
    OCSF_FTP_ACTIVITY = "OCSF.FtpActivity"
    OCSF_GROUP_MANAGEMENT = "OCSF.GroupManagement"
    OCSF_HTTP_ACTIVITY = "OCSF.HttpActivity"
    OCSF_INCIDENT_FINDING = "OCSF.IncidentFinding"
    OCSF_INVENTORY_INFO = "OCSF.InventoryInfo"
    OCSF_KERNEL_ACTIVITY = "OCSF.KernelActivity"
    OCSF_KERNEL_EXTENSION = "OCSF.KernelExtension"
    OCSF_MEMORY_ACTIVITY = "OCSF.MemoryActivity"
    OCSF_MODULE_ACTIVITY = "OCSF.ModuleActivity"
    OCSF_NETWORK_ACTIVITY = "OCSF.NetworkActivity"
    OCSF_NETWORK_FILE_ACTIVITY = "OCSF.NetworkFileActivity"
    OCSF_NTP_ACTIVITY = "OCSF.NtpActivity"
    OCSF_PATCH_STATE = "OCSF.PatchState"
    OCSF_PROCESS_ACTIVITY = "OCSF.ProcessActivity"
    OCSF_RDP_ACTIVITY = "OCSF.RdpActivity"
    OCSF_SCAN_ACTIVITY = "OCSF.ScanActivity"
    OCSF_SCHEDULED_JOB_ACTIVITY = "OCSF.ScheduledJobActivity"
    OCSF_SECURITY_FINDING = "OCSF.SecurityFinding"
    OCSF_SMB_ACTIVITY = "OCSF.SmbActivity"
    OCSF_SSH_ACTIVITY = "OCSF.SshActivity"
    OCSF_USER_ACCESS = "OCSF.UserAccess"
    OCSF_USER_INVENTORY = "OCSF.UserInventory"
    OCSF_VULNERABILITY_FINDING = "OCSF.VulnerabilityFinding"
    OCSF_WEB_RESOURCE_ACCESS_ACTIVITY = "OCSF.WebResourceAccessActivity"
    OCSF_WEB_RESOURCES_ACTIVITY = "OCSF.WebResourcesActivity"
    OCSF_WIN_PREFETCH_INFO = "OCSF.WinPrefetchInfo"
    OCSF_WIN_REGISTRY_KEY_ACTIVITY = "OCSF.WinRegistryKeyActivity"
    OCSF_WIN_REGISTRY_KEY_INFO = "OCSF.WinRegistryKeyInfo"
    OCSF_WIN_REGISTRY_VALUE_ACTIVITY = "OCSF.WinRegistryValueActivity"
    OCSF_WIN_REGISTRY_VALUE_INFO = "OCSF.WinRegistryValueInfo"
    OCSF_WIN_RESOURCE_ACTIVITY = "OCSF.WinResourceActivity"
    OCSF_WINPREFETCH_INFO = "OCSF.WinprefetchInfo"
    OCSF_WINREGISTRY_KEY_ACTIVITY = "OCSF.WinregistryKeyActivity"
    OCSF_WINREGISTRY_KEY_INFO = "OCSF.WinregistryKeyInfo"
    OCSF_WINREGISTRY_VALUE_ACTIVITY = "OCSF.WinregistryValueActivity"
    OCSF_WINREGISTRY_VALUE_INFO = "OCSF.WinregistryValueInfo"
    OCSF_WINRESOURCE_ACTIVITY = "OCSF.WinresourceActivity"
    OSSEC_EVENT_INFO = "OSSEC.EventInfo"
    OKTA_DEVICES = "Okta.Devices"
    OKTA_SYSTEM_LOG = "Okta.SystemLog"
    OKTA_USERS = "Okta.Users"
    ONELOGIN_EVENTS = "OneLogin.Events"
    ONEPASSWORD_AUDIT_EVENT = "OnePassword.AuditEvent"
    ONEPASSWORD_ITEM_USAGE = "OnePassword.ItemUsage"
    ONEPASSWORD_SIGNIN_ATTEMPT = "OnePassword.SignInAttempt"
    OSQUERY_BATCH = "Osquery.Batch"
    OSQUERY_DIFFERENTIAL = "Osquery.Differential"
    OSQUERY_SNAPSHOT = "Osquery.Snapshot"
    OSQUERY_STATUS = "Osquery.Status"
    PANTHER_AUDIT = "Panther.Audit"
    PROOFPOINT_EVENT = "Proofpoint.Event"
    PUSH_SECURITY_ACTIVITY = "PushSecurity.Activity"
    PUSH_SECURITY_ATTACK_DETECTION = "PushSecurity.AttackDetection"
    PUSH_SECURITY_CONTROLS = "PushSecurity.Controls"
    PUSH_SECURITY_ENTITIES = "PushSecurity.Entities"
    SALESFORCE_LOGIN = "Salesforce.Login"
    SALESFORCE_LOGIN_AS = "Salesforce.LoginAs"
    SALESFORCE_LOGOUT = "Salesforce.Logout"
    SALESFORCE_URI = "Salesforce.URI"
    SENTINELONE_ACTIVITY = "SentinelOne.Activity"
    SENTINELONE_DEEP_VISIBILITY = "SentinelOne.DeepVisibility"
    SENTINELONE_DEEP_VISIBILITY_V2 = "SentinelOne.DeepVisibilityV2"
    SLACK_ACCESS_LOGS = "Slack.AccessLogs"
    SLACK_AUDIT_LOGS = "Slack.AuditLogs"
    SLACK_INTEGRATION_LOGS = "Slack.IntegrationLogs"
    SNYK_GROUP_AUDIT = "Snyk.GroupAudit"
    SNYK_ORG_AUDIT = "Snyk.OrgAudit"
    SOPHOS_CENTRAL = "Sophos.Central"
    SURICATA_ALERT = "Suricata.Alert"
    SURICATA_ANOMALY = "Suricata.Anomaly"
    SURICATA_DHCP = "Suricata.DHCP"
    SURICATA_DNS = "Suricata.DNS"
    SURICATA_FILE_INFO = "Suricata.FileInfo"
    SURICATA_FLOW = "Suricata.Flow"
    SURICATA_HTTP = "Suricata.HTTP"
    SURICATA_SSH = "Suricata.SSH"
    SURICATA_TLS = "Suricata.TLS"
    SYSDIG_AUDIT = "Sysdig.Audit"
    SYSLOG_RFC3164 = "Syslog.RFC3164"
    SYSLOG_RFC5424 = "Syslog.RFC5424"
    TAILSCALE_AUDIT = "Tailscale.Audit"
    TAILSCALE_NETWORK = "Tailscale.Network"
    TENABLE_VULNERABILITY = "Tenable.Vulnerability"
    TINES_AUDIT = "Tines.Audit"
    TOR_EXIT_NODE = "Tor.ExitNode"
    TRAILDISCOVER_CLOUDTRAIL = "TrailDiscover.CloudTrail"
    WINDOWS_EVENT_LOGS = "Windows.EventLogs"
    WIZ_ISSUES = "Wiz.Issues"
    WORKDAY_ACTIVITY = "Workday.Activity"
    WORKDAY_SIGN_ON_ATTEMPT = "Workday.SignOnAttempt"
    ZEEK_CAPTURE_LOSS = "Zeek.CaptureLoss"
    ZEEK_CONN = "Zeek.Conn"
    ZEEK_DHCP = "Zeek.DHCP"
    ZEEK_DNS = "Zeek.DNS"
    ZEEK_DPD = "Zeek.DPD"
    ZEEK_FILES = "Zeek.Files"
    ZEEK_HTTP = "Zeek.HTTP"
    ZEEK_NTP = "Zeek.NTP"
    ZEEK_NOTICE = "Zeek.Notice"
    ZEEK_OCSP = "Zeek.OCSP"
    ZEEK_REPORTER = "Zeek.Reporter"
    ZEEK_SIP = "Zeek.SIP"
    ZEEK_SOFTWARE = "Zeek.Software"
    ZEEK_SSH = "Zeek.Ssh"
    ZEEK_SSL = "Zeek.Ssl"
    ZEEK_STATS = "Zeek.Stats"
    ZEEK_TUNNEL = "Zeek.Tunnel"
    ZEEK_WEIRD = "Zeek.Weird"
    ZEEK_X509 = "Zeek.X509"
    ZENDESK_AUDIT = "Zendesk.Audit"
    ZOOM_ACTIVITY = "Zoom.Activity"
    ZOOM_OPERATION = "Zoom.Operation"

    def __str__(self) -> str:
        """Returns a string representation of the class' value."""
        return self.value

    @staticmethod
    def get_attribute_name(value: str) -> str:
        """Returns the attribute name of the class' value."""
        for name, member in LogType.__members__.items():
            if member.value == value:
                return name
        raise AttributeError(f"LogType has no attribute with value {value}")



"""Global helpers for Auth0 detections"""

from pypanther.helpers.panther_base_helpers import deep_get


def auth0_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = deep_get(event, "data", "details", "request", "auth", "user", default="<NO_ACTOR_FOUND>")
    a_c["action"] = deep_get(event, "data", "description", default="<NO_ACTION_FOUND>")
    return a_c


def is_auth0_config_event(event):
    channel = deep_get(event, "data", "details", "request", "channel", default="")
    return channel == "https://manage.auth0.com/"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

box_access_granted_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="Access Granted",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "ACCESS_GRANTED",
            "source": {"id": "12345678", "type": "user", "login": "user@example", "name": "Bob Cat"},
        },
    ),
]


@panther_managed
class BoxAccessGranted(Rule):
    id = "Box.Access.Granted-prototype"
    display_name = "Box Access Granted"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box"]
    default_severity = Severity.LOW
    default_description = "A user granted access to their box account to Box technical support from account settings.\n"
    default_reference = (
        "https://support.box.com/hc/en-us/articles/7039943421715-Enabling-and-Disabling-Access-for-Box-Support"
    )
    default_runbook = "Investigate whether the user purposefully granted access to their account.\n"
    summary_attributes = ["p_any_ip_addresses"]
    tests = box_access_granted_tests

    def rule(self, event):
        return event.get("event_type") == "ACCESS_GRANTED"

    def title(self, event):
        return (
            f"User [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}] granted access to their account"
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

panther_saml_modified_tests: list[RuleTest] = [
    RuleTest(
        name="SAML config modified",
        expected_result=True,
        log={
            "actionName": "UPDATE_SAML_SETTINGS",
            "actionParams": {},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                "id": "111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
        },
    ),
    RuleTest(
        name="SAML config viewed",
        expected_result=False,
        log={
            "actionName": "GET_SAML_SETTINGS",
            "actionParams": {},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                "id": "111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
        },
    ),
]


@panther_managed
class PantherSAMLModified(Rule):
    id = "Panther.SAML.Modified-prototype"
    display_name = "Panther SAML configuration has been modified"
    log_types = [LogType.PANTHER_AUDIT]
    default_severity = Severity.HIGH
    tags = ["DataModel", "Defense Evasion:Impair Defenses"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_description = "An Admin has modified Panther's SAML configuration."
    default_runbook = "Ensure this change was approved and appropriate."
    default_reference = "https://docs.panther.com/system-configuration/saml"
    summary_attributes = ["p_any_ip_addresses", "p_any_usernames"]
    tests = panther_saml_modified_tests

    def rule(self, event):
        return event.get("actionName") == "UPDATE_SAML_SETTINGS" and event.get("actionResult") == "SUCCEEDED"

    def title(self, event):
        return f"Panther SAML config has been modified by {event.udm('actor_user')}"

    def alert_context(self, event):
        return {"user": event.udm("actor_user"), "ip": event.udm("source_ip")}



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

panther_user_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Admin Role Created",
        expected_result=False,
        log={
            "actionName": "CREATE_USER_ROLE",
            "actionParams": {
                "input": {
                    "logTypeAccessKind": "DENY_ALL",
                    "name": "New Admins",
                    "permissions": ["GeneralSettingsModify", "GeneralSettingsRead", "SummaryRead"],
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "1111111"},
                "id": "11111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
            "pantherVersion": "1.2.3",
            "sourceIP": "1.2.3.4",
            "timestamp": "2022-04-27 20:47:09.425",
        },
    ),
    RuleTest(
        name="Users's email was changed",
        expected_result=True,
        log={
            "XForwardedFor": ["1.2.3.4", "5.6.7.8"],
            "actionDescription": "Updates the information for a user",
            "actionName": "UPDATE_USER",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "email": "user-email+anyplus@springfield.gov",
                        "familyName": "Email",
                        "givenName": "User",
                        "id": "75757575-7575-7575-7575-757575757575",
                        "role": {"kind": "ID", "value": "(redacted)"},
                    },
                },
                "static": {},
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {
                    "email": "admin.email@springfield.gov",
                    "emailVerified": False,
                    "roleId": "89898989-8989-8989-8989-898989898989",
                    "roleName": "Admin",
                },
                "id": "PantherSSO_admin.email@springfield.gov",
                "name": "admin.email@springfield.gov",
                "type": "USER",
            },
            "p_any_ip_addresses": ["5.6.7.8", "1.2.3.4"],
            "p_any_trace_ids": ["PantherSSO_admin.email@springfield.gov"],
            "p_any_usernames": ["admin.email@springfield.gov"],
            "p_event_time": "2022-11-08 19:23:04.841",
            "p_log_type": "Panther.Audit",
            "p_parse_time": "2022-11-08 19:23:47.278",
            "p_row_id": "12341234123412341234123412341234",
            "p_source_id": "34343434-3434-3434-3434-343434343434",
            "p_source_label": "panther-audit-logs-region-name",
            "pantherVersion": "1.2.3",
            "sourceIP": "1.2.3.4",
            "timestamp": "2022-11-08 19:23:04.841",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
        },
    ),
    RuleTest(
        name="Users's role was changed",
        expected_result=True,
        log={
            "XForwardedFor": ["5.6.7.8", "1.2.3.4"],
            "actionDescription": "Updates the information for a user",
            "actionName": "UPDATE_USER",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "email": "user.email@springfield.gov",
                        "familyName": "Email",
                        "givenName": "User",
                        "id": "PantherSSO_user.email@springfield.gov",
                        "role": {"kind": "ID", "value": "(redacted)"},
                    },
                },
                "static": {},
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {
                    "email": "admin.email@springfield.gov",
                    "emailVerified": False,
                    "roleId": "12341234-1234-1234-1234-123412341234",
                    "roleName": "Admin",
                },
                "id": "PantherSSO_admin.email@springfield.gov",
                "name": "admin.email@springfield.gov",
                "type": "USER",
            },
            "p_any_ip_addresses": ["5.6.7.8", "1.2.3.4"],
            "p_any_trace_ids": ["PantherSSO_admin.email@springfield.gov"],
            "p_any_usernames": ["admin.email@springfield.gov"],
            "p_event_time": "2022-11-09 23:10:35.504",
            "p_log_type": "Panther.Audit",
            "p_parse_time": "2022-11-09 23:11:47.112",
            "p_row_id": "56785678567856785678567856785678",
            "p_source_id": "34563456-3456-3456-3456-345634563456",
            "p_source_label": "panther-audit-logs-region-name",
            "pantherVersion": "1.2.3",
            "sourceIP": "5.6.7.8",
            "timestamp": "2022-11-09 23:10:35.504",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
        },
    ),
    RuleTest(
        name="SCIM based user provision - INFO level",
        expected_result=True,
        log={
            "actionDescription": "User updated via SCIM",
            "actionName": "UPDATE_USER",
            "actionParams": {
                "input": {
                    "email": "user.email@springfield.gov",
                    "familyName": "",
                    "givenName": "",
                    "id": "PantherSSO_user.email@springfield.gov",
                    "requesterId": "00000000-0000-4000-8000-000000000000",
                    "roleId": None,
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {"id": "scim", "name": "scim", "type": "TOKEN"},
            "p_any_actor_ids": ["scim"],
            "p_any_usernames": ["scim"],
            "p_event_time": "2023-06-23 17:49:37.553847671",
            "p_log_type": "Panther.Audit",
            "p_parse_time": "2023-06-23 17:50:46.933652106",
            "p_source_label": "panther audit logs",
            "sourceIP": "12.12.12.12",
            "timestamp": "2023-06-23 17:49:37.553847671",
        },
    ),
    RuleTest(
        name="User modified by System account",
        expected_result=True,
        log={
            "actionDescription": "User updated automatically by SAML.",
            "actionName": "UPDATE_USER",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "email": "john.doe@usgs.gov",
                        "familyName": "Doe",
                        "givenName": "John",
                        "role": "AnalystReadOnly",
                    },
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {"id": "00000000-0000-4000-8000-000000000000", "name": "System", "type": "USER"},
            "p_log_type": "Panther.Audit",
            "pantherVersion": "1.86.15",
            "sourceIP": "",
            "timestamp": "2023-10-25 05:30:15.618835297",
        },
    ),
]


@panther_managed
class PantherUserModified(Rule):
    id = "Panther.User.Modified-prototype"
    display_name = "A User's Panther Account was Modified"
    log_types = [LogType.PANTHER_AUDIT]
    default_severity = Severity.HIGH
    tags = ["DataModel", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_description = (
        "A Panther user's role has been modified. This could mean password, email, or role has changed for the user."
    )
    default_runbook = "Validate that this user modification was intentional."
    default_reference = "https://docs.panther.com/panther-developer-workflows/api/operations/user-management"
    summary_attributes = ["p_any_ip_addresses"]
    tests = panther_user_modified_tests
    PANTHER_USER_ACTIONS = [event_type.USER_ACCOUNT_MODIFIED]

    def rule(self, event):
        if event.udm("event_type") not in self.PANTHER_USER_ACTIONS:
            return False
        return event.get("actionResult") == "SUCCEEDED"

    def title(self, event):
        change_target = deep_get(event, "actionParams", "dynamic", "input", "email")
        if change_target is None:
            change_target = deep_get(event, "actionParams", "input", "email")
        if change_target is None:
            change_target = deep_get(event, "actionParams", "email", default="<UNKNOWN_USER>")
        return f"The user account {change_target} was modified by {event.udm('actor_user')}"

    def alert_context(self, event):
        change_target = deep_get(event, "actionParams", "dynamic", "input", "email")
        if change_target is None:
            change_target = deep_get(event, "actionParams", "input", "email", default="<UNKNOWN_USER>")
        return {"user": event.udm("actor_user"), "change_target": change_target, "ip": event.udm("source_ip")}

    def severity(self, event):
        user = event.udm("actor_user")
        if user == "scim":
            return "INFO"
        if event.deep_get("actor", "id") == "00000000-0000-4000-8000-000000000000":
            return "INFO"
        return "DEFAULT"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_cloud_trail_root_access_key_created_tests: list[RuleTest] = [
    RuleTest(
        name="Root Access Key Created",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "CreateAccessKey",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": None,
            "responseElements": {
                "accessKey": {"accessKeyId": "1111", "createDate": "Jan 01, 2019 0:00:00 PM", "status": "Active"},
            },
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "signin.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "invokedBy": "signin.amazonaws.com",
                "principalId": "123456789012",
                "sessionContext": {"attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"}},
                "type": "Root",
            },
        },
    ),
    RuleTest(
        name="Root Created Access Key For User",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "CreateAccessKey",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"userName": "example-user"},
            "responseElements": {
                "accessKey": {
                    "accessKeyId": "1111",
                    "createDate": "Jan 01, 2019 0:00:00 PM",
                    "status": "Active",
                    "userName": "example-user",
                },
            },
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "signin.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "invokedBy": "signin.amazonaws.com",
                "principalId": "123456789012",
                "sessionContext": {"attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"}},
                "type": "Root",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailRootAccessKeyCreated(Rule):
    id = "AWS.CloudTrail.RootAccessKeyCreated-prototype"
    display_name = "Root Account Access Key Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity and Access Management", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_severity = Severity.CRITICAL
    default_description = "An access key was created for the Root account"
    default_runbook = "Verify that the root access key was created for legitimate reasons. If not, immediately revoke it and change the root login credentials. If it was created for legitimate reasons, monitor its use and ensure it is revoked when its need is gone.\n"
    default_reference = "https://docs.aws.amazon.com/general/latest/gr/managing-aws-access-keys.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_root_access_key_created_tests

    def rule(self, event):
        # Only check access key creation events
        if event.get("eventName") != "CreateAccessKey":
            return False
        # Only root can create root access keys
        if deep_get(event, "userIdentity", "type") != "Root":
            return False
        # Only alert if the root user is creating an access key for itself
        return event.get("requestParameters") is None

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_app_banner_acknowledged_tests: list[RuleTest] = [
    RuleTest(
        name="App Banner Acknowledged",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "action": "ACKNOWLEDGED",
                "appBanner": {
                    "buttonText": "Proceed anyway",
                    "mode": "ACKNOWLEDGE",
                    "subtext": "This app is not approved, please use Google Drive instead.",
                    "title": "This app is not approved for use",
                },
                "appType": "DROPBOX",
                "browser": "CHROME",
                "employee": {
                    "chatopsEnabled": True,
                    "creationTimestamp": 1698669223.0,
                    "department": "Security Engineering",
                    "email": "john.hill@example.com",
                    "firstName": "John",
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "lastName": "Hill",
                    "licensed": True,
                    "location": "New York",
                },
                "os": "WINDOWS",
                "sourceIpAddress": "8.158.25.38",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
            },
            "object": "APP_BANNER",
            "timestamp": 1698604061.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="App Banner Displayed",
        expected_result=False,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "action": "DISPLAYED",
                "appBanner": {
                    "buttonText": "Proceed anyway",
                    "mode": "ACKNOWLEDGE",
                    "subtext": "This app is not approved, please use Google Drive instead.",
                    "title": "This app is not approved for use",
                },
                "appType": "DROPBOX",
                "browser": "CHROME",
                "employee": {
                    "chatopsEnabled": True,
                    "creationTimestamp": 1698669223.0,
                    "department": "Security Engineering",
                    "email": "john.hill@example.com",
                    "firstName": "John",
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "lastName": "Hill",
                    "licensed": True,
                    "location": "New York",
                },
                "os": "WINDOWS",
                "sourceIpAddress": "8.158.25.38",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
            },
            "object": "APP_BANNER",
            "timestamp": 1698604061.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="App Banner Inform Mode",
        expected_result=False,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "action": "DISPLAYED",
                "appBanner": {
                    "buttonText": None,
                    "mode": "INFORM",
                    "subtext": "This app is not approved, please use Google Drive instead.",
                    "title": "This app is not approved for use",
                },
                "appType": "DROPBOX",
                "browser": "CHROME",
                "employee": {
                    "chatopsEnabled": True,
                    "creationTimestamp": 1698669223.0,
                    "department": "Security Engineering",
                    "email": "john.hill@example.com",
                    "firstName": "John",
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "lastName": "Hill",
                    "licensed": True,
                    "location": "New York",
                },
                "os": "WINDOWS",
                "sourceIpAddress": "8.158.25.38",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
            },
            "object": "APP_BANNER",
            "timestamp": 1698604061.0,
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityAppBannerAcknowledged(Rule):
    id = "Push.Security.App.Banner.Acknowledged-prototype"
    display_name = "Push Security App Banner Acknowledged"
    log_types = [LogType.PUSH_SECURITY_ACTIVITY]
    default_severity = Severity.LOW
    tests = push_security_app_banner_acknowledged_tests

    def rule(self, event):
        if event.get("object") != "APP_BANNER":
            return False
        if event.deep_get("new", "action") == "ACKNOWLEDGED":
            return True
        return False

    def title(self, event):
        app_type = event.deep_get("new", "appType")
        employee_email = event.deep_get("new", "employee", "email")
        return f"{app_type} accessed by {employee_email}"

    def alert_context(self, event):
        return {
            "Push Security app banner": event.deep_get("new", "appBanner", "mode"),
            "Title": event.deep_get("new", "appBanner", "title"),
            "Subtext": event.deep_get("new", "appBanner", "subtext"),
            "Button": event.deep_get("new", "appBanner", "buttonText"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awsec2_traffic_mirroring_tests: list[RuleTest] = [
    RuleTest(
        name="CreateTrafficMirrorFilter",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "a3c6297b-3320-4d32-b224-cc45ee75d561",
            "eventName": "CreateTrafficMirrorFilter",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 22:58:13",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123451234513"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 22:58:13",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:00:37.355",
            "p_row_id": "82670d2e7575bbd0e8fc97d014b5a80c",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123451234515",
            "requestID": "200a9157-dff7-4578-87d6-205b01d90a56",
            "requestParameters": {
                "CreateTrafficMirrorFilterRequest": {"ClientToken": "5b7eff74-2b70-4f92-8aa1-9c716bf151aa"},
            },
            "responseElements": {
                "CreateTrafficMirrorFilterResponse": {
                    "clientToken": "5b7eff74-2b70-4f92-8aa1-9c716bf151aa",
                    "requestId": "200a9157-dff7-4578-87d6-205b01d90a56",
                    "trafficMirrorFilter": {
                        "egressFilterRuleSet": "",
                        "ingressFilterRuleSet": "",
                        "networkServiceSet": "",
                        "tagSet": "",
                        "trafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                    },
                    "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123451234516",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="CreateTrafficMirrorFilterRule",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "61b6e2d6-788c-4b8d-9b6f-1ce17539e05e",
            "eventName": "CreateTrafficMirrorFilterRule",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 22:58:13",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123451234512"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 22:58:13",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:00:37.356",
            "p_row_id": "82670d2e7575bbd0e8fc97d014b6a80c",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123451234566",
            "requestID": "92220f15-633f-4b9c-a544-57b9b8228ec3",
            "requestParameters": {
                "CreateTrafficMirrorFilterRuleRequest": {
                    "ClientToken": "b59974fd-e63c-489a-9011-4386f541e2e7",
                    "DestinationCidrBlock": "0.0.0.0/0",
                    "Protocol": 6,
                    "RuleAction": "accept",
                    "RuleNumber": 100,
                    "SourceCidrBlock": "0.0.0.0/0",
                    "TrafficDirection": "egress",
                    "TrafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                },
            },
            "responseElements": {
                "CreateTrafficMirrorFilterRuleResponse": {
                    "clientToken": "b59974fd-e63c-489a-9011-4386f541e2e7",
                    "requestId": "92220f15-633f-4b9c-a544-57b9b8228ec3",
                    "trafficMirrorFilterRule": {
                        "destinationCidrBlock": "0.0.0.0/0",
                        "protocol": 6,
                        "ruleAction": "accept",
                        "ruleNumber": 100,
                        "sourceCidrBlock": "0.0.0.0/0",
                        "trafficDirection": "egress",
                        "trafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                        "trafficMirrorFilterRuleId": "tmfr-01669e70d44b9705a",
                    },
                    "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO:MakeStuffPublic",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="CreateTrafficMirrorSession",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "errorCode": "Client.NetworkInterfaceNotSupported",
            "errorMessage": "eni-0c61fbc84fca4138f must be attached to a supported instance",
            "eventCategory": "Management",
            "eventID": "5298b707-aa6b-4e4d-86c6-761ee19bb095",
            "eventName": "CreateTrafficMirrorSession",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 22:58:35",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 22:58:35",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:00:37.356",
            "p_row_id": "82670d2e7575bbd0e8fc97d014bba80c",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "6eda0d2f-6688-4877-ae3d-0d982d7e7aaf",
            "requestParameters": {
                "CreateTrafficMirrorSessionRequest": {
                    "ClientToken": "c773e3fd-0611-4fb0-a436-67fbef29031e",
                    "NetworkInterfaceId": "eni-0c61fbc84fca4138f",
                    "SessionNumber": 1,
                    "TrafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                    "TrafficMirrorTargetId": "tmt-0fd4b591901182794",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO:MakeStuffPublic",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="CreateTrafficMirrorTarget",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1ca9056c-efe7-4415-bb35-bbee893b9bd0",
            "eventName": "CreateTrafficMirrorTarget",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 23:05:21",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 23:05:21",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:07:31.858",
            "p_row_id": "de31c1202c72c6b6ff8698d014ba931a",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "4698f691-c987-412d-b11d-e4aa285394d1",
            "requestParameters": {
                "CreateTrafficMirrorTargetRequest": {
                    "ClientToken": "fc50f178-783d-4420-b788-345988244b83",
                    "NetworkInterfaceId": "eni-0fd6cc8547555878f",
                },
            },
            "responseElements": {
                "CreateTrafficMirrorTargetResponse": {
                    "clientToken": "fc50f178-783d-4420-b788-345988244b83",
                    "requestId": "4698f691-c987-412d-b11d-e4aa285394d1",
                    "trafficMirrorTarget": {
                        "networkInterfaceId": "eni-0fd6cc8547555878f",
                        "ownerId": 123123123123.0,
                        "tagSet": "",
                        "trafficMirrorTargetId": "tmt-0a45c694b91bcea54",
                        "type": "network-interface",
                    },
                    "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="DeleteTrafficMirrorTarget",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "2212e8b1-b053-4388-9d3a-1bf963b1f075",
            "eventName": "DeleteTrafficMirrorTarget",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 23:05:38",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 23:05:38",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:07:31.858",
            "p_row_id": "de31c1202c72c6b6ff8698d014bf931a",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "b56e2421-1fed-4052-b3ac-82daf374964c",
            "requestParameters": {
                "DeleteTrafficMirrorTargetRequest": {"TrafficMirrorTargetId": "tmt-0fd4b591901182794"},
            },
            "responseElements": {
                "DeleteTrafficMirrorTargetResponse": {
                    "requestId": "b56e2421-1fed-4052-b3ac-82daf374964c",
                    "trafficMirrorTargetId": "tmt-0fd4b591901182794",
                    "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="DescribeTrafficMirrorTargets",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "43b5d650-dfdf-41a7-9b57-831e5cf1d190",
            "eventName": "DescribeTrafficMirrorTargets",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 23:05:39",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 23:05:39",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:07:31.858",
            "p_row_id": "de31c1202c72c6b6ff8698d014c0931a",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": True,
            "recipientAccountId": "123123123123",
            "requestID": "6aa42f9d-b623-4d7e-b1b5-1647ac4d7f8e",
            "requestParameters": {"DescribeTrafficMirrorTargetsRequest": {"MaxResults": 1000}},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="ModifyTrafficMirrorSession",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "66c0fb61-804e-4cee-9f2e-c1d23c406a78",
            "eventName": "ModifyTrafficMirrorSession",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-11-15 23:10:17",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123451234513:role/MakeStuffPublic",
                "arn:aws:sts::123451234513:assumed-role/MakeStuffPublic",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA57JLR4M2ZZDJUXY3"],
            "p_any_usernames": ["MakeStuffPublic"],
            "p_event_time": "2022-11-15 23:10:17",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-15 23:13:49.967",
            "p_row_id": "9a217340fc13a7ff8b9b9ad014e0e205",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "a39291d0-81b6-470f-ae0d-0f028d5676ce",
            "requestParameters": {
                "ModifyTrafficMirrorSessionRequest": {
                    "RemoveField": [{"content": "description", "tag": 1}, {"content": "packet-length", "tag": 2}],
                    "SessionNumber": 2,
                    "TrafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                    "TrafficMirrorSessionId": "tms-05e1e21760419ecb6",
                    "TrafficMirrorTargetId": "tmt-0a45c694b91bcea54",
                    "VirtualNetworkId": 12348395.0,
                },
            },
            "responseElements": {
                "ModifyTrafficMirrorSessionResponse": {
                    "requestId": "a39291d0-81b6-470f-ae0d-0f028d5676ce",
                    "trafficMirrorSession": {
                        "networkInterfaceId": "eni-08dd7ebbda3b01770",
                        "ownerId": 123123123123.0,
                        "sessionNumber": 2,
                        "tagSet": "",
                        "trafficMirrorFilterId": "tmf-010db9a7d8056cc2d",
                        "trafficMirrorSessionId": "tms-05e1e21760419ecb6",
                        "trafficMirrorTargetId": "tmt-0a45c694b91bcea54",
                        "virtualNetworkId": 12348395.0,
                    },
                    "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                },
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA57JLR4M2ZZDJUXY3",
                "accountId": "123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/MakeStuffPublic",
                "principalId": "AROA57JLR4M2SBAPVC4BO",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-15T22:38:17Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/MakeStuffPublic",
                        "principalId": "AROA57JLR4M2SBAPVC4BO",
                        "type": "Role",
                        "userName": "MakeStuffPublic",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="UnrelatedEvent",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
            "eventName": "UpdateProjectVisibility",
            "eventSource": "codebuild.amazonaws.com",
            "eventTime": "2021-08-18T14:54:53Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "111122223333",
            "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
            "requestParameters": {
                "projectArn": "arn:aws:codebuild:us-east-1:111122223333:project/testproject1234",
                "projectVisibility": "PUBLIC_READ",
                "resourceAccessRole": "arn:aws:iam::111122223333:role/service-role/test",
            },
            "responseElements": None,
            "sourceIPAddress": "1.1.1.1",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
            "userIdentity": {
                "accessKeyId": "ASIAXXXXXXXXXXXX",
                "accountId": "111122223333",
                "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                "principalId": "111111111111",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="AWS Config Runs DescribeTrafficMirrorTargets",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "c98e81a7-15ee-4888-bbbb-cccccccccccc",
            "eventName": "DescribeTrafficMirrorTargets",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2023-02-10 18:08:33",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "123456789012",
            "requestID": "1531eeee-5ddd-4fff-8111-844444444444",
            "requestParameters": {"DescribeTrafficMirrorTargetsRequest": {"MaxResults": 1000}},
            "sourceIPAddress": "config.amazonaws.com",
            "userAgent": "config.amazonaws.com",
            "userIdentity": {
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/SomeAWSConfig/ConfigResourceCompositionSession",
                "invokedBy": "config.amazonaws.com",
                "principalId": "AROAAAAAAAAAAAAAAAAAA:ConfigResourceCompositionSession",
                "sessionContext": {
                    "attributes": {"creationDate": "2023-02-13T18:08:33Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/PantherAWSConfig",
                        "principalId": "AROAAAAAAAAAAAAAAAAAA",
                        "type": "Role",
                        "userName": "PantherAWSConfig",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSEC2TrafficMirroring(Rule):
    default_description = "This rule captures multiple traffic mirroring events in AWS Cloudtrail."
    display_name = "AWS EC2 Traffic Mirroring"
    default_reference = "https://attack.mitre.org/techniques/T1040/"
    default_runbook = "Examine other activities done by this user to determine whether or not activity is suspicious. If your network traffic is not encrypted, we recommend changing the severity to high or critical."
    default_severity = Severity.MEDIUM
    tags = ["AWS", "Cloudtrail", "MITRE"]
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Traffic.Mirroring-prototype"
    summary_attributes = ["userIdentity.type"]
    tests = awsec2_traffic_mirroring_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        event_names = [
            "CreateTrafficMirrorFilter",
            "CreateTrafficMirrorFilterRule",
            "CreateTrafficMirrorSession",
            "CreateTrafficMirrorTarget",
            "DeleteTrafficMirrorFilter",
            "DeleteTrafficMirrorFilterRule",
            "DeleteTrafficMirrorSession",
            "DeleteTrafficMirrorTarget",
            "DescribeTrafficMirrorFilters",
            "DescribeTrafficMirrorSessions",
            "DescribeTrafficMirrorTargets",
            "ModifyTrafficMirrorFilterNetworkServices",
            "ModifyTrafficMirrorFilterRule",
            "ModifyTrafficMirrorSession",
        ]
        if deep_get(event, "userIdentity", "invokedBy", default="").endswith(".amazonaws.com"):
            return False
        return event.get("eventSource", "") == "ec2.amazonaws.com" and event.get("eventName", "") in event_names

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method will
        # act as deduplication string.
        return f"{event.get('userIdentity', {}).get('arn', 'no-type')} ec2 activity found for {event.get('eventName')} in account {event.get('recipientAccountId')} in region {event.get('awsRegion')}."

    def dedup(self, event):
        #  (Optional) Return a string which will be used to deduplicate similar alerts.
        # Dedupe based on user identity, to not include multiple events from the same identity.
        return f"{event.get('userIdentity', {}).get('arn', 'no-user-identity-provided')}"

    def alert_context(self, event):
        #  (Optional) Return a dictionary with additional data to be included
        #  in the alert sent to the SNS/SQS/Webhook destination
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_created_tests: list[RuleTest] = [
    RuleTest(
        name="CloudTrail Was Created",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "Tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventName": "CreateTrail",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="KMS Decrypt Event",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "111:panther-snapshot-scheduler",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "Decrypt",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "encryptionContext": {
                    "aws:lambda:FunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:test-function",
                },
            },
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                },
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Error Creating CloudTrail",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "CloudTrailInvalidClientTokenIdException",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "Tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventName": "CreateTrail",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSCloudTrailCreated(Rule):
    id = "AWS.CloudTrail.Created-prototype"
    display_name = "A CloudTrail Was Created or Updated"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Discovery:Cloud Service Dashboard"]
    reports = {"CIS": ["3.5"], "MITRE ATT&CK": ["TA0007:T1538"]}
    default_severity = Severity.INFO
    default_description = "A CloudTrail Trail was created, updated, or enabled.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-cloudtrail-modified"
    default_reference = (
        "https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-create-and-update-a-trail.html"
    )
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_created_tests
    # API calls that are indicative of CloudTrail changes
    CLOUDTRAIL_CREATE_UPDATE = {"CreateTrail", "UpdateTrail", "StartLogging"}

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.CLOUDTRAIL_CREATE_UPDATE

    def title(self, event):
        return f"CloudTrail [{deep_get(event, 'requestParameters', 'name')}] was created/updated"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import github_alert_context

github_public_repository_created_tests: list[RuleTest] = [
    RuleTest(
        name="Public Repo Created",
        expected_result=True,
        log={
            "_document_id": "abCD",
            "action": "repo.create",
            "actor": "example-actor",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 22:40:20.268",
            "created_at": "2022-12-11 22:40:20.268",
            "org": "example-io",
            "repo": "example-io/oops",
            "visibility": "public",
        },
    ),
    RuleTest(
        name="Private Repo Created",
        expected_result=False,
        log={
            "_document_id": "abCD",
            "action": "repo.create",
            "actor": "example-actor",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 22:40:20.268",
            "created_at": "2022-12-11 22:40:20.268",
            "org": "example-io",
            "repo": "example-io/oops",
            "visibility": "private",
        },
    ),
]


@panther_managed
class GithubPublicRepositoryCreated(Rule):
    default_description = "A public Github repository was created."
    display_name = "Github Public Repository Created"
    default_runbook = "Confirm this github repository was intended to be created as 'public' versus 'private'."
    default_reference = "https://docs.github.com/en/get-started/quickstart/create-a-repo"
    default_severity = Severity.MEDIUM
    tags = ["Github Repository", "Public", "Repository Created"]
    log_types = [LogType.GITHUB_AUDIT]
    id = "Github.Public.Repository.Created-prototype"
    summary_attributes = ["actor", "repository", "visibility"]
    tests = github_public_repository_created_tests
    # def dedup(event):
    #  (Optional) Return a string which will be used to deduplicate similar alerts.
    # return ''

    def rule(self, event):
        # Return True if a public repository was created
        return event.get("action", "") == "repo.create" and event.get("visibility", "") == "public"

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] created with public status by Github user [{event.get('actor')}]."

    def alert_context(self, event):
        #  (Optional) Return a dictionary with additional data to be included in the alert
        # sent to the SNS/SQS/Webhook destination
        return github_alert_context(event)



"""Environment variables like ORG_ID, folder id/name mappings"""

PRODUCTION_PROJECT_IDS = ["example-production", "example-platform"]
ORG_ID = "888888888888"

rule_exceptions = {
    "gcp_k8s_exec_into_pod": {
        "allowed_principals": [
            {
                "principals": ["system:serviceaccount:example-namespace:example-namespace-service-account"],
                # If empty, then all namespaces
                "namespaces": [],
                # If projects empty then all projects
                "projects": [],
            },
            {
                "principals": ["example-allowed-user@example.com"],
                "namespaces": ["istio-system"],
                "projects": [],
            },
        ],
    },
}



import json

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

duo_admin_lockout_tests: list[RuleTest] = [
    RuleTest(
        name="Admin lockout- invalid json",
        expected_result=True,
        log={
            "action": "admin_lockout",
            "description": '"message": "Admin temporarily locked out due to too many passcode attempts."',
            "isotimestamp": "2022-12-14 21:02:03",
            "timestamp": "2022-12-14 21:02:03",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Admin lockout- valid json",
        expected_result=True,
        log={
            "action": "admin_lockout",
            "description": '{"message": "Admin temporarily locked out due to too many passcode attempts."}',
            "isotimestamp": "2022-12-14 21:02:03",
            "timestamp": "2022-12-14 21:02:03",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Bypass Create",
        expected_result=False,
        log={
            "action": "bypass_create",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminLockout(Rule):
    default_description = "Alert when a duo administrator is locked out of their account."
    display_name = "Duo Admin Lockout"
    default_reference = "https://duo.com/docs/adminapi"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.Lockout-prototype"
    tests = duo_admin_lockout_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "admin_lockout"

    def title(self, event):
        # If no 'dedup' function is defined, the return value
        # of this method will act as deduplication string.
        try:
            desc = json.loads(event.get("description", {}))
            message = desc.get("message", "<NO_MESSAGE_FOUND>")[:-1]
        except ValueError:
            message = "Invalid Json"
        return f"Duo Admin [{event.get('username', '<NO_USER_FOUND>')}] is locked out. Reason: [{message}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

wiz_alert_passthrough_tests: list[RuleTest] = [
    RuleTest(
        name="Open Alert",
        expected_result=True,
        log={
            "createdAt": "2024-06-04 02:28:06.763277000",
            "entitySnapshot": {
                "cloudProviderURL": "",
                "externalId": "someExternalId",
                "id": "12345",
                "name": "someName",
                "nativeType": "",
                "providerId": "someProviderId",
                "region": "",
                "resourceGroupExternalId": "",
                "subscriptionExternalId": "",
                "subscriptionName": "",
                "tags": {},
                "type": "DATA_FINDING",
            },
            "id": "54321",
            "notes": [],
            "projects": [
                {
                    "businessUnit": "",
                    "id": "45678",
                    "name": "Project 2",
                    "riskProfile": {"businessImpact": "MBI"},
                    "slug": "project-2",
                },
            ],
            "serviceTickets": [],
            "severity": "HIGH",
            "sourceRule": {
                "__typename": "Control",
                "controlDescription": "Alert Description",
                "id": "12345",
                "name": "Alert Name",
                "resolutionRecommendation": "Alert Resolution Recommendation",
                "securitySubCategories": [
                    {
                        "category": {"framework": {"name": "Wiz for Risk Assessment"}, "name": "High Profile Threats"},
                        "title": "High-profile vulnerability exploited in the wild",
                    },
                    {
                        "category": {"framework": {"name": "MITRE ATT&CK Matrix"}, "name": "TA0001 Initial Access"},
                        "title": "T1190 Exploit Public-Facing Application",
                    },
                ],
            },
            "status": "OPEN",
            "statusChangedAt": "2024-06-04 02:28:06.597355000",
            "type": "TOXIC_COMBINATION",
            "updatedAt": "2024-06-04 02:28:06.763277000",
        },
    ),
    RuleTest(
        name="Resolved Alert",
        expected_result=False,
        log={
            "createdAt": "2024-06-04 02:28:06.763277000",
            "entitySnapshot": {
                "cloudProviderURL": "",
                "externalId": "someExternalId",
                "id": "12345",
                "name": "someName",
                "nativeType": "",
                "providerId": "someProviderId",
                "region": "",
                "resourceGroupExternalId": "",
                "subscriptionExternalId": "",
                "subscriptionName": "",
                "tags": {},
                "type": "DATA_FINDING",
            },
            "id": "54321",
            "notes": [],
            "projects": [
                {
                    "businessUnit": "",
                    "id": "45678",
                    "name": "Project 2",
                    "riskProfile": {"businessImpact": "MBI"},
                    "slug": "project-2",
                },
            ],
            "serviceTickets": [],
            "severity": "HIGH",
            "sourceRule": {
                "__typename": "Control",
                "controlDescription": "Alert Description",
                "id": "12345",
                "name": "Alert Name",
                "resolutionRecommendation": "Alert Resolution Recommendation",
            },
            "status": "RESOLVED",
            "statusChangedAt": "2024-06-04 02:28:06.597355000",
            "type": "TOXIC_COMBINATION",
            "updatedAt": "2024-06-04 02:28:06.763277000",
        },
    ),
]


@panther_managed
class WizAlertPassthrough(Rule):
    id = "Wiz.Alert.Passthrough-prototype"
    default_description = "This rule enriches and contextualizes security alerts generated by Wiz."
    display_name = "Wiz Alert Passthrough Rule"
    default_runbook = "Review the Wiz alert details to determine what malicious behavior was detected, and whether or not it was blocked."
    default_reference = "https://www.wiz.io/product"
    default_severity = Severity.MEDIUM
    log_types = [LogType.WIZ_ISSUES]
    tests = wiz_alert_passthrough_tests

    def rule(self, event):
        return event.get("status") == "OPEN"

    def title(self, event):
        return f"[Wiz Alert]: {event.deep_get('sourceRule', 'name', default='ALERT_DESCRIPTION_NOT_FOUND')}"

    def severity(self, event):
        return event.get("severity")

    def dedup(self, event):
        return event.get("id")

    def description(self, event):
        return event.deep_get("sourceRule", "controlDescription", default="<DESCRIPTION_NOT_FOUND>")

    def runbook(self, event):
        return event.deep_get("sourceRule", "resolutionRecommendation", default="<RECOMMENDATION_NOT_FOUND>")

    def alert_context(self, event):
        security_subcategories = event.deep_get("sourceRule", "securitySubCategories", default=[{}])
        return {
            "id": event.get("id", "<ID_NOT_FOUND>"),
            "type": event.get("type", "<TYPE_NOT_FOUND>"),
            "entity_snapshot": event.get("entitySnapshot", {}),
            "mitre_attack_categories": [
                subcategory
                for subcategory in security_subcategories
                if deep_get(subcategory, "category", "framework", "name") == "MITRE ATT&CK Matrix"
            ],
        }



from typing import Annotated, Hashable, List, TypeVar

import annotated_types
from pydantic import AfterValidator, Field
from pydantic_core import PydanticCustomError

T = TypeVar("T", bound=Hashable)


def _validate_unique_list(v: list[T]) -> list[T]:
    if len(v) != len(set(v)):
        raise PydanticCustomError("unique_list", "List must be unique")
    return v


UniqueList = Annotated[List[T], AfterValidator(_validate_unique_list), Field(json_schema_extra={"uniqueItems": True})]
NonEmptyUniqueList = Annotated[
    List[T],
    annotated_types.Len(1, None),
    AfterValidator(_validate_unique_list),
    Field(json_schema_extra={"uniqueItems": True}),
]



import abc
import copy
import dataclasses
from enum import Enum
from typing import List, Optional

from pypanther import LogType
from pypanther.utils import try_asdict

"""This file contains data model definitions for the PyPanther package.
It is still in development and is subject to change.
"""


class FieldType(str, Enum):
    """Enumeration of all possible field types."""

    STRING = "string"
    INT = "int"
    SMALL_INT = "smallint"
    BIG_INT = "bigint"
    FLOAT = "float"
    BOOLEAN = "boolean"
    TIMESTAMP = "timestamp"
    ARRAY = "array"
    OBJECT = "object"
    JSON = "json"


_FIELD_MAPPING_ALL_ATTRIBUTES = [
    "log_type",
    "field_path",
]


@dataclasses.dataclass
class FieldMapping:
    """Represents a field mapping in a data model."""

    # TODO: Right now we support field_paths but we should be able to support transformations as well
    # TODO: This could be pointing to a schema attribute in the log type

    log_type: LogType | str
    """The log type that this field belongs to."""

    field_path: str
    """The path to the field in the log."""

    def asdict(self):
        """Returns a dictionary representation of the instance."""
        return {key: try_asdict(getattr(self, key)) for key in _FIELD_MAPPING_ALL_ATTRIBUTES if hasattr(self, key)}


_FIELD_ALL_ATTRIBUTES = [
    "name",
    "type",
    "mappings",
    "description",
]


@dataclasses.dataclass
class Field:
    """Represents a field in a data model."""

    name: str
    """The name of the field. This is the key that will be used to access the field in the data model."""

    type: FieldType
    """The type of the field."""

    mappings: List[FieldMapping]
    """Mappings describe how the data model field is derived from the various log types."""

    description: str = ""
    """A description of the field."""

    def asdict(self):
        """Returns a dictionary representation of the instance."""
        return {key: try_asdict(getattr(self, key)) for key in _FIELD_ALL_ATTRIBUTES if hasattr(self, key)}


_DATA_MODEL_ALL_ATTRS = [
    "description",
    "enabled",
    "fields",
]


class DataModel(abc.ABC):
    """A Panther data model. This class should be subclassed to create a new Data Model."""

    description: str = ""
    """A description of the data model."""

    enabled: bool = True
    """Whether the data model is enabled can can be used."""

    fields: List[Field]
    """The fields that make up the data model."""

    @classmethod
    def is_panther_managed(cls) -> bool:
        return cls.__module__.startswith("pypanther")

    @classmethod
    def override(
        cls,
        description: Optional[str] = None,
        enabled: Optional[bool] = None,
        fields: Optional[List[Field]] = None,
    ):
        for key, val in locals().items():
            if key == "cls":
                continue

            if val is not None:
                setattr(cls, key, val)

    def __init_subclass__(cls, **kwargs):
        """
        Creates a copy of all class attributes to avoid modifications affecting parent.fields.
        child.fields.append(Field("foo",...))
        parent.fields.append(Field("foo",...) # not inherited by children of parent
        """
        for attr in _DATA_MODEL_ALL_ATTRS:
            if attr not in cls.__dict__:
                try:
                    v = getattr(cls, attr)
                except AttributeError:
                    v = None

                if v is not None:
                    setattr(cls, attr, copy.deepcopy(v))
        super().__init_subclass__(**kwargs)

    @classmethod
    def asdict(cls):
        """Returns a dictionary representation of the class."""
        return {key: try_asdict(getattr(cls, key)) for key in _DATA_MODEL_ALL_ATTRS if hasattr(cls, key)}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gc_piamservice_account_keyscreate_tests: list[RuleTest] = [
    RuleTest(
        name="privilege-escalation",
        expected_result=True,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": True, "permission": "iam.serviceAccountKeys.create"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    RuleTest(
        name="fail",
        expected_result=False,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": False, "permission": "iam.serviceAccountKeys.create"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
]


@panther_managed
class GCPiamserviceAccountKeyscreate(Rule):
    id = "GCP.iam.serviceAccountKeys.create-prototype"
    display_name = "GCP.Iam.ServiceAccountKeys.Create"
    default_description = "If your user is assigned a custom IAM role, then iam.roles.update will allow you to update the “includedPermissons” on that role. Because it is assigned to you, you will gain the additional privileges, which could be anything you desire."
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_reference = "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"
    default_runbook = "Confirm this was authorized and necessary behavior. This is not a vulnerability in GCP, it is a vulnerability in how GCP environment is configured, so it is necessary to be aware of these attack vectors and to defend against them. It’s also important to remember that privilege escalation does not necessarily need to pass through the IAM service to be effective. Make sure to follow the principle of least-privilege in your environments to help mitigate these security risks."
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    tests = gc_piamservice_account_keyscreate_tests

    def rule(self, event):
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "iam.serviceAccountKeys.create" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context

slack_audit_logs_user_privilege_changed_to_user_tests: list[RuleTest] = [
    RuleTest(
        name="Role Changed to User",
        expected_result=True,
        log={
            "action": "role_change_to_user",
            "actor": {
                "type": "user",
                "user": {
                    "email": "slack-enterprise-example@example.io",
                    "id": "W015MH5MPGE",
                    "name": "primary-owner",
                    "team": "T017E0M3CQ4",
                },
            },
            "context": {
                "ip_address": "12.12.12.12",
                "location": {
                    "domain": "example-workspace-domain",
                    "id": "T017E0M3CQ4",
                    "name": "example-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
            },
            "date_create": "2023-02-24 18:34:18",
            "entity": {
                "type": "user",
                "user": {
                    "email": "example-account@example.com",
                    "id": "U04R70MM40K",
                    "name": "Example Account",
                    "team": "T017E0M3CQ4",
                },
            },
            "id": "4c248a02-119c-4f76-ba5d-a96767d45be8",
        },
    ),
    RuleTest(
        name="Role Changed to Admin",
        expected_result=False,
        log={
            "action": "role_change_to_admin",
            "actor": {
                "type": "user",
                "user": {
                    "email": "slack-enterprise-example@example.io",
                    "id": "W015MH5MPGE",
                    "name": "primary-owner",
                    "team": "T017E0M3CQ4",
                },
            },
            "context": {
                "ip_address": "12.12.12.12",
                "location": {
                    "domain": "example-workspace-domain",
                    "id": "T017E0M3CQ4",
                    "name": "example-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
            },
            "date_create": "2023-02-24 18:33:21",
            "entity": {
                "type": "user",
                "user": {
                    "email": "example-account@example.com",
                    "id": "U04R70MM40K",
                    "name": "Example Account",
                    "team": "T017E0M3CQ4",
                },
            },
            "id": "1ad8fa51-f18e-450a-8e18-cfe31278be96",
        },
    ),
]


@panther_managed
class SlackAuditLogsUserPrivilegeChangedToUser(Rule):
    id = "Slack.AuditLogs.UserPrivilegeChangedToUser-prototype"
    display_name = "Slack User Privileges Changed to User"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Impact", "Account Access Removal"]
    reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack account is changed to User from an elevated role."
    default_reference = "https://slack.com/intl/en-gb/help/articles/360018112273-Types-of-roles-in-Slack"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_user_privilege_changed_to_user_tests

    def rule(self, event):
        return event.get("action") == "role_change_to_user"

    def title(self, event):
        username = deep_get(event, "entity", "user", "name", default="<unknown-entity>")
        email = deep_get(event, "entity", "user", "email", default="<unknown-email>")
        return f"Slack {username}'s ({email}) role changed to User"

    def alert_context(self, event):
        return slack_alert_context(event)



"""Used to define global helpers and variables for Cloudflare events."""

from typing import Any

#
# Source mapping values from
# https://developers.cloudflare.com/logs/reference/log-fields/zone/firewall_events/
FIREWALL_SOURCE_MAPPING = {
    "unknown": "Unknown",
    "asn": "ASN",
    "country": "Country",
    "ip": "IP",
    "iprange": "IP Range",
    "securitylevel": "Security Level",
    "zonelockdown": "Zone Lockdown",
    "waf": "WAF",
    "firewallrules": "Firewall Rules",
    "uablock": "User Agent",
    "ratelimit": "Rate Limit",
    "bic": "Browser Integrity Check",
    "hot": "Hotlink Protection",
    "l7ddos": "L7 DDoS",
    "botfight": "Bot Fight (Classic)",
    "validation": "Invalid",
    "apishield": "API Shield",
    "botmanagement": "Bot Management",
    "dlp": "Data Loss Prevention",
    "firewallmanaged": "Firewall Managed Rules",
    "firewallcustom": "Firewall Custom Rulesets",
}


# Historical usage of map_source_to_name had detections
# passing in event.get('Source'), hence the Any input
def map_source_to_name(event: Any) -> str:
    if isinstance(event, str):
        return FIREWALL_SOURCE_MAPPING.get(event.lower(), event)
    return FIREWALL_SOURCE_MAPPING.get(event.get("Source", "").lower(), event.get("Source", "<NO_SOURCE>"))


def cloudflare_fw_alert_context(event: dict = None):
    keep_keys = [
        "Action",
        "ClientIP",
        "ClientRequestHost",
        "Datetime",
        "EdgeColoCode",
        "EdgeResponseStatus",
        "Kind",
        "RuleID",
        "Source",
    ]
    context_dict = {}
    for k in keep_keys:
        context_dict[k] = event.get(k, f"<{k}_NOT_IN_EVENT>")
    context_dict["pan_cf_source"] = map_source_to_name(event)
    return context_dict


def cloudflare_http_alert_context(event: dict = None):
    keep_keys = [
        "BotScore",
        "BotScoreSrc",
        "CacheCacheStatus",
        "CacheResponseStatus",
        "ClientIP",
        "ClientDeviceType",
        "ClientRequestHost",
        "ClientRequestMethod",
        "ClientRequestPath",
        "EdgeResponseStatus",
    ]
    context_dict = {}
    for k in keep_keys:
        if k in event:
            context_dict[k] = event.get(k, f"<{k}_NOT_IN_EVENT>")
    return context_dict



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

decoy_iam_assumed_tests: list[RuleTest] = [
    RuleTest(
        name="IAM-Decoy-Assumed",
        expected_result=True,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "AssumeRole",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "11.1.111.11",
                        "Organization": {},
                    },
                    "ServiceName": "sts.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 13:17:15.000000000",
            "Description": "Private decoy IAM role arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f was accessed by arn:aws:iam::123456789012:user/tester. This IAM role has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "sts.amazonaws.com",
            "Id": "1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T13:17:21.469Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd123-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "AWS Signin, aws-internal/3 aws-sdk-java/1.12.720 Linux/5.10.215-181.850.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/17.0.11+10-LTS java/17.0.11 kotlin/1.3.72 vendor/Amazon.com_Inc. cfg/retry-mode/standard cfg/auth-source#unknown",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab9-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsIamRole",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFGH0TOGJSGNQKI0:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-24T22:32:38Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "ABCDEFGH0TOGJSGNQKI0",
                                    "Type": "Role",
                                    "UserName": "user_ab21cde50f",
                                },
                            },
                        },
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Suspicious activity detected accessing private decoy IAM role arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 13:17:15.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:34:07.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:35:04.272574202",
            "p_row_id": "zjj8nmnw9f90uulxfa3bmen8rv5stlcx",
            "p_schema_version": 0,
            "p_source_id": "bb4e16c5-43dd-450c-9227-39f0d152659c",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
    RuleTest(
        name="IAM-Decoy-Not-Assumed",
        expected_result=False,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "ListRoles",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "99.6.134.57",
                        "Organization": {},
                    },
                    "ServiceName": "sts.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 13:17:15.000000000",
            "Description": "Private decoy IAM role arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f was not accessed by arn:aws:iam::123456789012:user/tester. This IAM role has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "sts.amazonaws.com",
            "Id": "1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T13:17:21.469Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd123-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "AWS Signin, aws-internal/3 aws-sdk-java/1.12.720 Linux/5.10.215-181.850.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/17.0.11+10-LTS java/17.0.11 kotlin/1.3.72 vendor/Amazon.com_Inc. cfg/retry-mode/standard cfg/auth-source#unknown",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab9-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsIamRole",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFGH0TOGJSGNQKI0:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-24T22:32:38Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "ABCDEFGH0TOGJSGNQKI0",
                                    "Type": "Role",
                                    "UserName": "user_ab21cde50f",
                                },
                            },
                        },
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Non-Suspicious activity detected accessing private decoy IAM role arn:aws:iam::123456789012:role/Dummy-Test-InfoRole-ab21cde50f",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 13:17:15.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:34:07.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:35:04.272574202",
            "p_row_id": "zjj8nmnw9f90uulxfa3bmen8rv5stlcx",
            "p_schema_version": 0,
            "p_source_id": "bb4e16c5-43dd-450c-9227-39f0d152659c",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
]


@panther_managed
class DecoyIAMAssumed(Rule):
    id = "Decoy.IAM.Assumed-prototype"
    display_name = "Decoy IAM Assumed"
    enabled = False
    log_types = [LogType.AWS_SECURITY_FINDING_FORMAT]
    default_severity = Severity.HIGH
    default_description = "Actor assumed decoy IAM role"
    default_reference = "https://aws.amazon.com/blogs/security/how-to-detect-suspicious-activity-in-your-aws-account-by-using-private-decoy-resources/"
    inline_filters = [{"All": []}]
    tests = decoy_iam_assumed_tests

    def rule(self, event):
        # List of suspicious API events
        # NOTE: There may be more API events that's not listed
        suspicious_api_events = ["AssumeRole"]
        # Return True if the API value is in the list of suspicious API events
        if event["GeneratorId"] == "sts.amazonaws.com":
            # Extract the API value from the event
            api_value = event["Action"]["AwsApiCallAction"]["Api"]
            return api_value in suspicious_api_events
        return False

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method will act as dedup string.
        # NOTE: Not sure if the offending actor Id will always be in the 0th index of Resources
        # It's possible to just return the Title as a whole string
        secret = event["Resources"][0]["Id"]
        return f"Suspicious activity detected accessing private decoy IAM role {secret}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_advanced_protection_tests: list[RuleTest] = [
    RuleTest(
        name="Advanced Protection Enabled",
        expected_result=False,
        log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
            "type": "titanium_change",
            "name": "titanium_enroll",
        },
    ),
    RuleTest(
        name="Advanced Protection Disabled",
        expected_result=True,
        log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
            "type": "titanium_change",
            "name": "titanium_unenroll",
        },
    ),
]


@panther_managed
class GSuiteAdvancedProtection(Rule):
    id = "GSuite.AdvancedProtection-prototype"
    display_name = "GSuite User Advanced Protection Change"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Defense Evasion:Impair Defenses"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.LOW
    default_description = "A user disabled advanced protection for themselves.\n"
    default_reference = "https://support.google.com/a/answer/9378686?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Have the user re-enable Google Advanced Protection\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_advanced_protection_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "user_accounts":
            return False
        return bool(event.get("name") == "titanium_unenroll")

    def title(self, event):
        return f"Advanced protection was disabled for user [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk, okta_alert_context

okta_new_behavior_accessing_admin_console_tests: list[RuleTest] = [
    RuleTest(
        name="New Behavior Accessing Admin Console (behavior)",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "device": {"name": "Evil Computer"},
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "behaviors": {
                        "New Geo-Location=NEGATIVE": None,
                        "New Device=POSITIVE": None,
                        "New IP=POSITIVE": None,
                        "New State=NEGATIVE": None,
                        "New Country=NEGATIVE": None,
                        "Velocity=NEGATIVE": None,
                        "New City=NEGATIVE": None,
                    },
                },
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "policy.evaluate_sign_on",
            "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="New Behavior Accessing Admin Console (logSecurityDataOnly)",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "device": {"name": "Evil Computer"},
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "logOnlySecurityData": {
                        "risk": {"level": "LOW"},
                        "behaviors": {
                            "New Geo-Location": "NEGATIVE",
                            "New Device": "POSITIVE",
                            "New IP": "POSITIVE",
                            "New State": "NEGATIVE",
                            "New Country": "NEGATIVE",
                            "Velocity": "NEGATIVE",
                            "New City": "NEGATIVE",
                        },
                    },
                },
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "policy.evaluate_sign_on",
            "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="Not New Behavior",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "logOnlySecurityData": {
                        "risk": {"level": "LOW"},
                        "behaviors": {
                            "New Geo-Location": "NEGATIVE",
                            "New Device": "NEGATIVE",
                            "New IP": "NEGATIVE",
                            "New State": "NEGATIVE",
                            "New Country": "NEGATIVE",
                            "Velocity": "NEGATIVE",
                            "New City": "NEGATIVE",
                        },
                    },
                },
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "policy.evaluate_sign_on",
            "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaNewBehaviorAccessingAdminConsole(Rule):
    id = "Okta.New.Behavior.Accessing.Admin.Console-prototype"
    display_name = "Okta New Behaviors Acessing Admin Console"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0001:T1078.004"]}
    default_severity = Severity.HIGH
    default_description = "New Behaviors Observed while Accessing Okta Admin Console. A user attempted to access the Okta Admin Console from a new device with a new IP.\n"
    default_runbook = "Configure Authentication Policies (Application Sign-on Policies) for access to privileged applications, including the Admin Console, to require re-authentication “at every sign-in”. Turn on and test New Device and Suspicious Activity end-user notifications.\n"
    default_reference = "https://sec.okta.com/articles/2023/08/cross-tenant-impersonation-prevention-and-detection\n"
    tests = okta_new_behavior_accessing_admin_console_tests

    def rule(self, event):
        if event.get("eventtype") != "policy.evaluate_sign_on":
            return False
        if "Okta Admin Console" not in deep_walk(event, "target", "displayName", default=""):
            return False
        behaviors = deep_get(event, "debugContext", "debugData", "behaviors")
        if behaviors:
            return "New Device=POSITIVE" in behaviors and "New IP=POSITIVE" in behaviors
        return (
            deep_get(event, "debugContext", "debugData", "logOnlySecurityData", "behaviors", "New Device") == "POSITIVE"
            and deep_get(event, "debugContext", "debugData", "logOnlySecurityData", "behaviors", "New IP") == "POSITIVE"
        )

    def title(self, event):
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> accessed Okta Admin Console using new behaviors: New IP: {deep_get(event, 'client', 'ipAddress', default='<ipAddress-not-found>')} New Device: {deep_get(event, 'device', 'name', default='<deviceName-not-found>')}"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_device_unlock_failure_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Mobile Event",
        expected_result=False,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_SYNC_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    RuleTest(
        name="Small Number of Failed Logins",
        expected_result=False,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "FAILED_PASSWD_ATTEMPTS": 2},
        },
    ),
    RuleTest(
        name="Multiple Failed Login Attempts with int Type",
        expected_result=True,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "FAILED_PASSWD_ATTEMPTS": 100},
        },
    ),
    RuleTest(
        name="Multiple Failed Login Attempts with String Type",
        expected_result=True,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "FAILED_PASSWD_ATTEMPTS": "100"},
        },
    ),
]


@panther_managed
class GSuiteDeviceUnlockFailure(Rule):
    id = "GSuite.DeviceUnlockFailure-prototype"
    display_name = "GSuite User Device Unlock Failures"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Credential Access:Brute Force"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_severity = Severity.MEDIUM
    default_description = "Someone failed to unlock a user's device multiple times in quick succession.\n"
    default_reference = "https://support.google.com/a/answer/6350074?hl=en"
    default_runbook = "Verify that these unlock attempts came from the user, and not a malicious actor which has acquired the user's device.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_device_unlock_failure_tests
    MAX_UNLOCK_ATTEMPTS = 10

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        if event.get("name") == "FAILED_PASSWORD_ATTEMPTS_EVENT":
            attempts = deep_get(event, "parameters", "FAILED_PASSWD_ATTEMPTS")
            return int(attempts if attempts else 0) > self.MAX_UNLOCK_ATTEMPTS
        return False

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]'s device had multiple failed unlock attempts"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION

zendesk_sensitive_data_redaction_off_tests: list[RuleTest] = [
    RuleTest(
        name="Zendesk - Credit Card Redaction Off",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Credit Card Redaction",
            "action": "create",
            "change_description": "Disabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Credit Card Redaction On",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Credit Card Redaction",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="User assumption settings changed",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Account Assumption",
            "action": "update",
            "change_description": "Changed",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskSensitiveDataRedactionOff(Rule):
    id = "Zendesk.SensitiveDataRedactionOff-prototype"
    display_name = "Zendesk Credit Card Redaction Off"
    log_types = [LogType.ZENDESK_AUDIT]
    tags = ["Zendesk", "Collection:Data from Information Repositories"]
    reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    default_severity = Severity.HIGH
    default_description = "A user updated account setting that disabled credit card redaction."
    default_runbook = "Re-enable credit card redaction."
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408822124314-Automatically-redacting-credit-card-numbers-from-tickets"
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_sensitive_data_redaction_off_tests
    REDACTION_ACTIONS = {"create", "destroy"}

    def rule(self, event):
        return (
            event.get("source_type") == "account_setting"
            and event.get("action", "") in self.REDACTION_ACTIONS
            and (event.get("source_label", "") == "Credit Card Redaction")
        )

    def title(self, event):
        action = event.get(ZENDESK_CHANGE_DESCRIPTION, "<UNKNOWN_ACTION>")
        return f"User [{event.udm('actor_user')}] {action} credit card redaction"

    def severity(self, event):
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() != "disabled":
            return "INFO"
        return "HIGH"



from typing import Final

VERSION_STRING: Final = "0.48.0"
CONFIG_FILE = ".panther_settings.yml"

# The UserID is required by Panther for some API calls, but we have no way of
# acquiring it, and it isn't used for anything. This is a valid UUID used by the
# Panther deployment tool to indicate this action was performed automatically.
PANTHER_USER_ID = "00000000-0000-4000-8000-000000000000"


class ReplayStatus:
    DONE = "DONE"
    CANCELED = "CANCELED"
    ERROR_EVALUATION = "ERROR_EVALUATION"
    ERROR_COMPUTATION = "ERROR_COMPUTATION"
    EVALUATION_IN_PROGRESS = "EVALUATION_IN_PROGRESS"
    COMPUTATION_IN_PROGRESS = "COMPUTATION_IN_PROGRESS"


ENABLE_CORRELATION_RULES_FLAG = "EnableCorrelationRules"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

cisco_umbrella_dns_suspicious_tests: list[RuleTest] = [
    RuleTest(
        name="Suspicious Domain",
        expected_result=True,
        log={
            "action": "Allow",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "cron.photoscape.ch.",
        },
    ),
    RuleTest(
        name="Safe Domain",
        expected_result=False,
        log={
            "action": "Allowed",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "google.com.",
        },
    ),
]


@panther_managed
class CiscoUmbrellaDNSSuspicious(Rule):
    id = "CiscoUmbrella.DNS.Suspicious-prototype"
    display_name = "Cisco Umbrella Suspicious Domains"
    enabled = False
    dedup_period_minutes = 480
    log_types = [LogType.CISCO_UMBRELLA_DNS]
    tags = ["DNS", "Configuration Required"]
    default_reference = "https://umbrella.cisco.com/blog/abcs-of-dns"
    default_severity = Severity.LOW
    default_description = "Monitor suspicious or known malicious domains"
    default_runbook = "Inspect the domain and check the host for other indicators of compromise"
    summary_attributes = ["action", "internalIp", "externalIp", "domain", "responseCode"]
    tests = cisco_umbrella_dns_suspicious_tests
    DOMAINS_TO_MONITOR = {"photoscape.ch"}  # Sample malware domain

    def rule(self, event):
        return any(domain in event.get("domain") for domain in self.DOMAINS_TO_MONITOR)

    def title(self, event):
        return "Suspicious lookup to domain " + event.get("domain", "<UNKNOWN_DOMAIN>")



from typing import Iterable, List, Set, Type

from pydantic import NonNegativeInt, PositiveInt

from pypanther.base import Rule
from pypanther.data_models_v2 import DataModel
from pypanther.severity import Severity
from pypanther.unit_tests import RuleTest
from pypanther.utils import filter_iterable_by_kwargs

_RULE_REGISTRY: Set[Type[Rule]] = set()
_DATA_MODEL_REGISTRY: Set[Type[DataModel]] = set()


def register(arg: Type[Rule] | Type[DataModel] | Iterable[Type[Rule] | Type[DataModel]]) -> None:
    """The register function is used to register rules and data models with the pypanther library."""
    if _register_rule(arg):  # type: ignore
        return
    if _register_data_model(arg):  # type: ignore
        return

    try:
        it = iter(arg)  # type: ignore
    except TypeError:
        raise ValueError(f"argument must be a Rule or DataModel or an iterable of them not {arg}")

    for e in it:
        if _register_rule(e):  # type: ignore
            continue
        if _register_data_model(e):  # type: ignore
            continue
        raise ValueError(f"argument must be a Rule or DataModel or an iterable of them not {arg}")


def _register_rule(rule: Type[Rule]) -> bool:
    """Register a rule with the pypanther library. Returns True if the rule was registered, False otherwise."""
    if isinstance(rule, type) and issubclass(rule, Rule):
        rule.validate()
        _RULE_REGISTRY.add(rule)
        return True
    return False


def _register_data_model(dm: Type[DataModel]) -> bool:
    """
    Register a data model with the pypanther library.
    Returns True if the data model was registered, False otherwise.
    """
    if isinstance(dm, type) and issubclass(dm, DataModel):
        _DATA_MODEL_REGISTRY.add(dm)
        return True
    return False


def registered_rules(
    log_types: List[str] | None = None,
    id: str | None = None,
    create_alert: bool | None = None,
    dedup_period_minutes: NonNegativeInt | None = None,
    display_name: str | None = None,
    enabled: bool | None = None,
    scheduled_queries: List[str] | None = None,
    summary_attributes: List[str] | None = None,
    tests: List[RuleTest] | None = None,
    threshold: PositiveInt | None = None,
    tags: List[str] | None = None,
    reports: dict[str, List[str]] | None = None,
    default_severity: Severity | None = None,
    default_description: str | None = None,
    default_reference: str | None = None,
    default_runbook: str | None = None,
    default_destinations: List[str] | None = None,
) -> Set[Type[Rule]]:
    filters = locals()
    return set(
        filter_iterable_by_kwargs(
            _RULE_REGISTRY,
            **filters,
        ),
    )


def registered_data_models() -> Set[Type[DataModel]]:
    return _DATA_MODEL_REGISTRY



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_saml_created_tests: list[RuleTest] = [
    RuleTest(
        name="SAML Auth Connector modified",
        expected_result=True,
        log={
            "cluster_name": "teleport.example.com",
            "code": "T8200I",
            "ei": 0,
            "event": "saml.created",
            "name": "okta",
            "time": "2023-09-19 18:00:00",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "max.mustermann@zumbeispiel.example",
        },
    ),
]


@panther_managed
class TeleportSAMLCreated(Rule):
    id = "Teleport.SAMLCreated-prototype"
    display_name = "A SAML Connector was created or modified"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.HIGH
    default_description = "A SAML connector was created or modified"
    reports = {"MITRE ATT&CK": ["TA0042:T1585"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "When a SAML connector is modified, it can potentially change the trust model of the Teleport Cluster. Validate that these changes were expected and correct.\n"
    summary_attributes = ["event", "code", "user", "name"]
    tests = teleport_saml_created_tests

    def rule(self, event):
        return event.get("event") == "saml.created"

    def title(self, event):
        return f"A SAML connector was created or updated by User [{event.get('user', '<UNKNOWN_USER>')}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_phishing_attack_tests: list[RuleTest] = [
    RuleTest(
        name="Phishing Detected - Block Mode",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "appType": "OKTA",
                "browser": "CHROME",
                "employee": {
                    "chatopsEnabled": True,
                    "creationTimestamp": 1698669223.0,
                    "department": "Security Engineering",
                    "email": "john.hill@example.com",
                    "firstName": "John",
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "lastName": "Hill",
                    "licensed": True,
                    "location": "New York",
                },
                "mode": "BLOCK",
                "os": "WINDOWS",
                "referrerUrl": "https://statics.teams.cdn.office.net/",
                "sourceIpAddress": "8.158.25.38",
                "url": "https://evil.com/okta.php",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
            },
            "object": "PASSWORD_PHISHING",
            "timestamp": 1698604061.0,
            "version": "1",
        },
    ),
    RuleTest(
        name="Phishing Detected - Monitor Mode",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "appType": "OKTA",
                "browser": "CHROME",
                "employee": {
                    "chatopsEnabled": True,
                    "creationTimestamp": 1698669223.0,
                    "department": "Security Engineering",
                    "email": "john.hill@example.com",
                    "firstName": "John",
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "lastName": "Hill",
                    "licensed": True,
                    "location": "New York",
                },
                "mode": "MONITOR",
                "os": "WINDOWS",
                "referrerUrl": "https://statics.teams.cdn.office.net/",
                "sourceIpAddress": "8.158.25.38",
                "url": "https://evil.com/okta.php",
                "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
            },
            "object": "PASSWORD_PHISHING",
            "timestamp": 1698604061.0,
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityPhishingAttack(Rule):
    id = "Push.Security.Phishing.Attack-prototype"
    display_name = "Push Security Phishing Attack"
    log_types = [LogType.PUSH_SECURITY_CONTROLS]
    default_severity = Severity.HIGH
    tests = push_security_phishing_attack_tests

    def rule(self, event):
        if event.get("object") == "PASSWORD_PHISHING":
            return True
        return False

    def severity(self, event):
        if event.deep_get("new", "mode") != "BLOCK":
            return "HIGH"
        return "LOW"

    def title(self, event):
        app_type = event.deep_get("new", "appType")
        employee_email = event.deep_get("new", "employee", "email")
        new_mode = event.deep_get("new", "mode")
        return (
            f"Phishing attack on app {app_type} user {employee_email}.              Attack detected in mode {new_mode}."
        )
